# frozen_string_literal: true

require 'singleton'
require 'openssl'

module  Apiphobic
module  Tokens
  class Configuration
    include Singleton

    attr_writer :available_audiences,
                :available_issuers,
                :available_roles,
                :available_subjects,
                :default_audience,
                :default_availability_leeway_in_seconds,
                :default_expiration_in_minutes,
                :default_expiration_leeway_in_seconds,
                :default_issuer,
                :default_roles,
                :default_subject,
                :private_key,
                :type

    def available_audiences
      @available_audiences || []
    end

    def available_issuers
      @available_issuers || []
    end

    def available_roles
      @available_roles || []
    end

    def available_subjects
      @available_subjects || []
    end

    def default_audience
      @default_audience || 'User'
    end

    def default_availability_leeway_in_seconds
      @default_availability_leeway_in_seconds || 5
    end

    def default_expiration_in_minutes
      @default_expiration_in_minutes || (7 * 24 * 60)
    end

    def default_expiration_leeway_in_seconds
      @default_expiration_leeway_in_seconds || 5
    end

    def default_issuer
      @default_issuer || 'apiphobic'
    end

    def default_roles
      @default_roles || %w{standard}
    end

    def default_subject
      @default_subject || nil
    end

    def private_key
      return unless @private_key
      return @private_key if @private_key.respond_to?(:sign)

      OpenSSL::PKey::RSA.new(@private_key)
    end

    def to_h
      {
        available_audiences:                    available_audiences,
        available_issuers:                      available_issuers,
        available_roles:                        available_roles,
        available_subjects:                     available_subjects,
        default_audience:                       default_audience,
        default_availability_leeway_in_seconds: default_availability_leeway_in_seconds,
        default_expiration_in_minutes:          default_expiration_in_minutes,
        default_expiration_leeway_in_seconds:   default_expiration_leeway_in_seconds,
        default_issuer:                         default_issuer,
        default_roles:                          default_roles,
        default_subject:                        default_subject,
        private_key:                            private_key,
        type:                                   type,
      }
    end

    def type
      @type || 'JWE'
    end
  end

  def self.configure
    yield configuration
  end

  def self.configuration
    Configuration.instance
  end
end
end
